=== modified file 'NEWS'
--- NEWS	2009-01-15 09:11:40 +0000
+++ NEWS	2009-01-15 15:27:22 +0000
@@ -48,6 +48,9 @@
 
   INTERNALS:
 
+    * ``TreeDelta.show()`` now accepts a ``filter`` parameter allowing log
+      formatters to retrict the output.
+      (Vincent Ladeuil)
 
 bzr 1.11rc1 "Eyes up!" 2009-01-09
 ---------------------------------

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2008-12-01 16:51:55 +0000
+++ bzrlib/delta.py	2008-12-10 09:33:06 +0000
@@ -26,7 +26,7 @@
 class TreeDelta(object):
     """Describes changes from one tree to another.
 
-    Contains four lists:
+    Contains seven lists:
 
     added
         (path, id, kind)
@@ -34,12 +34,14 @@
         (path, id, kind)
     renamed
         (oldpath, newpath, id, kind, text_modified, meta_modified)
+    kind_changed
+        (path, id, old_kind, new_kind)
     modified
         (path, id, kind, text_modified, meta_modified)
     unchanged
         (path, id, kind)
     unversioned
-        (path, kind)
+        (path, None, kind)
 
     Each id is listed only once.
 
@@ -107,93 +109,104 @@
 
 
     def show(self, to_file, show_ids=False, show_unchanged=False,
-             short_status=False, indent=''):
-        """output this delta in status-like form to to_file."""
-        def show_list(files, short_status_letter=''):
-            for item in files:
-                path, fid, kind = item[:3]
-
-                if kind == 'directory':
-                    path += '/'
-                elif kind == 'symlink':
-                    path += '@'
-
-                if len(item) == 5 and item[4]:
-                    path += '*'
-
-                if show_ids:
-                    to_file.write(indent + '%s  %-30s %s\n' % (short_status_letter,
-                        path, fid))
+             short_status=False, indent='',
+             filter=None):
+        """Output this delta in status-like form to to_file.
+
+        :param to_file: A file-like object where the output is displayed.
+
+        :param show_ids: Output the file ids if True.
+
+        :param show_unchanged: Output the unchanged files if True.
+
+        :param short_status: Single-line status if True.
+
+        :param indent: Added at the beginning of all output lines (for merged
+            revisions).
+
+        :param filter: A callable receiving a path and a file id and
+            returning True if the path should be displayed.
+        """
+
+        def decorate_path(path, kind, meta_modified=None):
+            if kind == 'directory':
+                path += '/'
+            elif kind == 'symlink':
+                path += '@'
+            if meta_modified:
+                path += '*'
+            return path
+
+        def show_more_renamed(item):
+            (oldpath, file_id, kind,
+             text_modified, meta_modified, newpath) = item
+            dec_new_path = decorate_path(newpath, kind, meta_modified)
+            to_file.write(' => %s' % dec_new_path)
+            if text_modified or meta_modified:
+                extra_modified.append((newpath, file_id, kind,
+                                       text_modified, meta_modified))
+
+        def show_more_kind_changed(item):
+            (path, file_id, old_kind, new_kind) = item
+            to_file.write(' (%s => %s)' % (old_kind, new_kind))
+
+        def show_path(path, file_id, kind, meta_modified,
+                      default_format, with_file_id_format):
+            dec_path = decorate_path(path, kind, meta_modified)
+            if show_ids:
+                to_file.write(with_file_id_format % dec_path)
+            else:
+                to_file.write(default_format % dec_path)
+
+        def show_list(files, long_status_name, short_status_letter,
+                      default_format='%s', with_file_id_format='%-30s',
+                      show_more=None):
+            if files:
+                header_shown = False
+                if short_status:
+                    prefix = short_status_letter
                 else:
-                    to_file.write(indent + '%s  %s\n' % (short_status_letter, path))
-
-        if self.removed:
-            if not short_status:
-                to_file.write(indent + 'removed:\n')
-                show_list(self.removed)
-            else:
-                show_list(self.removed, 'D')
-
-        if self.added:
-            if not short_status:
-                to_file.write(indent + 'added:\n')
-                show_list(self.added)
-            else:
-                show_list(self.added, 'A')
-
+                    prefix = ''
+                prefix = indent + prefix + '  '
+
+                for item in files:
+                    path, file_id, kind = item[:3]
+                    if (filter is not None and not filter(path, file_id)):
+                        continue
+                    if not header_shown and not short_status:
+                        to_file.write(indent + long_status_name + ':\n')
+                        header_shown = True
+                    meta_modified = None
+                    if len(item) == 5:
+                        meta_modified = item[4]
+
+                    to_file.write(prefix)
+                    show_path(path, file_id, kind, meta_modified,
+                              default_format, with_file_id_format)
+                    if show_more is not None:
+                        show_more(item)
+                    if show_ids:
+                        to_file.write(' %s' % file_id)
+                    to_file.write('\n')
+
+        show_list(self.removed, 'removed', 'D')#
+        show_list(self.added, 'added', 'A')
         extra_modified = []
-
-        if self.renamed:
-            short_status_letter = 'R'
-            if not short_status:
-                to_file.write(indent + 'renamed:\n')
-                short_status_letter = ''
-            for (oldpath, newpath, fid, kind,
-                 text_modified, meta_modified) in self.renamed:
-                if text_modified or meta_modified:
-                    extra_modified.append((newpath, fid, kind,
-                                           text_modified, meta_modified))
-                if meta_modified:
-                    newpath += '*'
-                if show_ids:
-                    to_file.write(indent + '%s  %s => %s %s\n' % (
-                        short_status_letter, oldpath, newpath, fid))
-                else:
-                    to_file.write(indent + '%s  %s => %s\n' % (
-                        short_status_letter, oldpath, newpath))
-
-        if self.kind_changed:
-            if short_status:
-                short_status_letter = 'K'
-            else:
-                to_file.write(indent + 'kind changed:\n')
-                short_status_letter = ''
-            for (path, fid, old_kind, new_kind) in self.kind_changed:
-                if show_ids:
-                    suffix = ' '+fid
-                else:
-                    suffix = ''
-                to_file.write(indent + '%s  %s (%s => %s)%s\n' % (
-                    short_status_letter, path, old_kind, new_kind, suffix))
-
-        if self.modified or extra_modified:
-            short_status_letter = 'M'
-            if not short_status:
-                to_file.write(indent + 'modified:\n')
-                short_status_letter = ''
-            show_list(self.modified, short_status_letter)
-            show_list(extra_modified, short_status_letter)
-            
-        if show_unchanged and self.unchanged:
-            if not short_status:
-                to_file.write(indent + 'unchanged:\n')
-                show_list(self.unchanged)
-            else:
-                show_list(self.unchanged, 'S')
-
-        if self.unversioned:
-            to_file.write(indent + 'unknown:\n')
-            show_list(self.unversioned)
+        # Reorder self.renamed tuples so that all lists share the same
+        # order for their 3 first fields and that they also begin like
+        # the self.modified tuples
+        renamed = [(p, i, k, tm, mm, np)
+                   for  p, np, i, k, tm, mm  in self.renamed]
+        show_list(renamed, 'renamed', 'R', with_file_id_format='%s',
+                  show_more=show_more_renamed)
+        show_list(self.kind_changed, 'kind changed', 'K',
+                  with_file_id_format='%s',
+                  show_more=show_more_kind_changed)
+        show_list(self.modified + extra_modified, 'modified', 'M')
+        if show_unchanged:
+            show_list(self.unchanged, 'unchanged', 'S')
+
+        show_list(self.unversioned, 'unknown', ' ')
 
     def get_changes_as_text(self, show_ids=False, show_unchanged=False,
              short_status=False):

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2008-12-11 03:18:52 +0000
+++ bzrlib/merge.py	2009-01-14 12:27:02 +0000
@@ -1090,7 +1090,7 @@
                                 parent_trans_id, trans_id)
 
     def merge_contents(self, file_id):
-        """Performa a merge on file_id contents."""
+        """Performs a merge on file_id contents."""
         def contents_pair(tree):
             if file_id not in tree:
                 return (None, None)

=== modified file 'bzrlib/tests/test_delta.py'
--- bzrlib/tests/test_delta.py	2007-03-07 23:15:10 +0000
+++ bzrlib/tests/test_delta.py	2008-12-10 09:33:06 +0000
@@ -15,10 +15,11 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
-from StringIO import StringIO
+from cStringIO import StringIO
 
 from bzrlib import (
     delta as _mod_delta,
+    revision as _mod_revision,
     tests,
     )
 
@@ -187,7 +188,7 @@
                            exe_change=False)
 
 
-class TestChangesFrom (tests.TestCaseWithTransport):
+class TestChangesFrom(tests.TestCaseWithTransport):
 
     def show_string(self, delta, *args,  **kwargs):
         to_file = StringIO()
@@ -238,3 +239,83 @@
                            True, False)], delta.renamed)
         self.assertTrue(delta.has_changed())
         self.assertTrue(delta.touches_file_id('file-id'))
+
+
+class TestDeltaShow(tests.TestCaseWithTransport):
+
+    def _get_delta(self):
+        # We build the delta from a real tree to avoid depending on internal
+        # implementation details.
+        wt = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/f1', '1\n'),
+                                  ('branch/f2', '2\n'),
+                                  ('branch/f3', '3\n'),
+                                  ('branch/f4', '4\n'),
+                                  ('branch/dir/',),
+                                 ])
+        wt.add(['f1', 'f2', 'f3', 'f4', 'dir'],
+               ['f1-id', 'f2-id', 'f3-id', 'f4-id', 'dir-id'])
+        wt.commit('commit one', rev_id='1')
+
+        long_status = """added:
+  dir/
+  f1
+  f2
+  f3
+  f4
+"""
+        short_status = """A  dir/
+A  f1
+A  f2
+A  f3
+A  f4
+"""
+
+        repo = wt.branch.repository
+        d = wt.changes_from(repo.revision_tree(_mod_revision.NULL_REVISION))
+        return d, long_status, short_status
+
+    def test_delta_show_short_status_no_filter(self):
+        d, long_status, short_status = self._get_delta()
+        out = StringIO()
+        d.show(out, short_status=True)
+        self.assertEquals(short_status, out.getvalue())
+
+    def test_delta_show_long_status_no_filter(self):
+        d, long_status, short_status = self._get_delta()
+        out = StringIO()
+        d.show(out, short_status=False)
+        self.assertEquals(long_status, out.getvalue())
+
+    def test_delta_show_no_filter(self):
+        d, long_status, short_status = self._get_delta()
+        out = StringIO()
+        def not_a_filter(path, file_id):
+            return True
+        d.show(out, short_status=True, filter=not_a_filter)
+        self.assertEquals(short_status, out.getvalue())
+
+    def test_delta_show_short_status_single_file_filter(self):
+        d, long_status, short_status = self._get_delta()
+        out = StringIO()
+        def only_f2(path, file_id):
+            return path == 'f2'
+        d.show(out, short_status=True, filter=only_f2)
+        self.assertEquals("A  f2\n", out.getvalue())
+
+    def test_delta_show_long_status_single_file_filter(self):
+        d, long_status, short_status = self._get_delta()
+        out = StringIO()
+        def only_f2(path, file_id):
+            return path == 'f2'
+        d.show(out, short_status=False, filter=only_f2)
+        self.assertEquals("added:\n  f2\n", out.getvalue())
+
+    def test_delta_show_short_status_single_file_id_filter(self):
+        d, long_status, short_status = self._get_delta()
+        out = StringIO()
+        def only_f2_id(path, file_id):
+            return file_id == 'f2-id'
+        d.show(out, short_status=True, filter=only_f2_id)
+        self.assertEquals("A  f2\n", out.getvalue())
+

